==> eg_particle_interactions-v0.8.1-fabric-mc1.21.8-sources.jar: games/enchanted/eg_particle_interactions/common/particle/falling_spin/FallingSpinningParticle.java <==
package games.enchanted.eg_particle_interactions.common.particle.falling_spin;

import games.enchanted.eg_particle_interactions.common.duck.ParticleAccess;
import games.enchanted.eg_particle_interactions.common.mixin.accessor.client.ParticleAccessor;
import games.enchanted.eg_particle_interactions.common.util.MathHelpers;
import net.minecraft.class_2400;
import net.minecraft.class_3532;
import net.minecraft.class_3999;
import net.minecraft.class_4002;
import net.minecraft.class_4003;
import net.minecraft.class_638;
import net.minecraft.class_703;
import net.minecraft.class_707;
import net.minecraft.client.particle.*;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

public class FallingSpinningParticle extends class_4003 {
    private float rotSpeed;
    protected float spinAcceleration;
    protected float maxSpinSpeed = 1f;
    private boolean transparency;

    protected FallingSpinningParticle(class_638 level, double x, double y, double z, double xSpeed, double ySpeed, double zSpeed, class_4002 spriteSet, float gravityMultiplier) {
        super(level, x, y, z);
        this.method_18141(spriteSet.method_18139(this.field_3840));
        this.field_3844 = class_3532.method_32750(this.field_3840, 0.25F, 0.38F);;
        this.field_28786 = 1.0F;
        this.field_3852 = xSpeed + (Math.random() * 2.0 - 1.0) * 0.05000000074505806;
        this.field_3869 = ySpeed + (Math.random() * 2.0 - 1.0) * 0.05000000074505806;
        this.field_3850 = zSpeed + (Math.random() * 2.0 - 1.0) * 0.05000000074505806;
        this.field_3847 = (int)(16.0 / ((double)this.field_3840.method_43057() * 0.8 + 0.2)) + 2;
        this.spinAcceleration = (float)Math.toRadians(this.field_3840.method_43056() ? -5.0 : 5.0);
        this.field_3839 = (float) Math.toRadians(this.field_3840.method_39332(0, 360));
        this.field_3857 = this.field_3839;

        float particleSize = this.field_3840.method_43056() ? 0.07F : 0.08F;
        this.field_17867 = particleSize;
        this.method_3080(particleSize, particleSize);
        this.field_3844 *= gravityMultiplier;

        this.transparency = false;

        ((ParticleAccess) this).eg_particle_interactions$setBypassMovementCollisionCheck(true);
    }
    protected FallingSpinningParticle(class_638 level, double x, double y, double z, double xSpeed, double ySpeed, double zSpeed, class_4002 spriteSet, float gravityMultiplier, boolean transparency) {
        this(level, x, y, z, xSpeed, ySpeed, zSpeed, spriteSet, gravityMultiplier);
        this.transparency = transparency;
    }

    @Override
    public void method_3070() {
        this.rotSpeed += this.rotSpeed >= this.maxSpinSpeed ? 0 : (this.spinAcceleration / 2.0f);
        if(this.rotSpeed > this.maxSpinSpeed) this.rotSpeed = this.maxSpinSpeed;

        this.field_3857 = this.field_3839;
        if( !this.field_3845 && !((ParticleAccessor) this).block_place_particle$getStoppedByCollision() ) {
            this.field_3839 += this.rotSpeed / 6.5f;
        }

        this.field_3852 *= 0.949999988079071;
        this.field_3869 *= 0.8999999761581421;
        this.field_3850 *= 0.949999988079071;

        // if moving downwards
        if(this.field_3869 < 0 && ((ParticleAccess) this).eg_particle_interactions$getBypassMovementCollisionCheck()) {
            ((ParticleAccess) this).eg_particle_interactions$setBypassMovementCollisionCheck(false);
        }

        super.method_3070();
    }

    @Override
    public @NotNull class_3999 method_18122() {
        return this.transparency ? class_3999.field_17829 : class_3999.field_17828;
    }

    public static class GenericLeafProvider implements class_707<class_2400> {
        private final class_4002 spriteSet;

        public GenericLeafProvider(class_4002 spriteSet) {
            this.spriteSet = spriteSet;
        }

        @Nullable
        @Override
        public class_703 createParticle(@NotNull class_2400 type, @NotNull class_638 level, double x, double y, double z, double xSpeed, double ySpeed, double zSpeed) {
            return new FallingSpinningParticle(level, x, y, z, xSpeed, ySpeed, zSpeed, spriteSet, 1);
        }
    }

    public static class SnowflakeProvider implements class_707<class_2400> {
        private final class_4002 spriteSet;

        public SnowflakeProvider(class_4002 spriteSet) {
            this.spriteSet = spriteSet;
        }

        @Nullable
        @Override
        public class_703 createParticle(@NotNull class_2400 type, @NotNull class_638 level, double x, double y, double z, double xSpeed, double ySpeed, double zSpeed) {
            FallingSpinningParticle particle = new FallingSpinningParticle(level, x, y, z, xSpeed, ySpeed, zSpeed, spriteSet, 0.4f, true);
            float particleSize = level.field_9229.method_43056() ? 0.10F : 0.12F;
            particle.field_17867 = particleSize;
            particle.method_3080(particleSize, particleSize);
            particle.spinAcceleration = 0;
            particle.maxSpinSpeed = 0;
            particle.field_3839 = 0;
            particle.field_3857 = 0;
            return particle;
        }
    }

    public static class RandomisedSizeMoreGravityProvider implements class_707<class_2400> {
        private final class_4002 spriteSet;

        public RandomisedSizeMoreGravityProvider(class_4002 spriteSet) {
            this.spriteSet = spriteSet;
        }

        @Nullable
        @Override
        public class_703 createParticle(@NotNull class_2400 type, @NotNull class_638 level, double x, double y, double z, double xSpeed, double ySpeed, double zSpeed) {
            FallingSpinningParticle particle = new FallingSpinningParticle(level, x, y, z, xSpeed, ySpeed, zSpeed, spriteSet, 2f);
            float particleSize = MathHelpers.randomBetween(0.08f, 0.12f);
            particle.field_17867 = particleSize;
            particle.method_3080(particleSize, particleSize);
            return particle;
        }
    }

    public static class PaleOakProvider implements class_707<class_2400> {
        private final class_4002 spriteSet;

        public PaleOakProvider(class_4002 spriteSet) {
            this.spriteSet = spriteSet;
        }

        @Nullable
        @Override
        public class_703 createParticle(@NotNull class_2400 type, @NotNull class_638 level, double x, double y, double z, double xSpeed, double ySpeed, double zSpeed) {
            FallingSpinningParticle particle = new FallingSpinningParticle(level, x, y, z, xSpeed, ySpeed, zSpeed, spriteSet, 0.6f);
            float particleSize = level.field_9229.method_43056() ? 0.1f : 0.15f;
            particle.field_17867 = particleSize;
            particle.method_3080(particleSize, particleSize);
            particle.maxSpinSpeed = 0.1f;
            particle.spinAcceleration = (float)Math.toRadians(level.field_9229.method_43056() ? -1.0 : 1.0);
            return particle;
        }
    }
}

==> eg_particle_interactions-v0.8.3-fabric-mc1.21.10-sources.jar: games/enchanted/eg_particle_interactions/common/particle/falling_spin/FallingSpinningParticle.java <==
package games.enchanted.eg_particle_interactions.common.particle.falling_spin;

import games.enchanted.eg_particle_interactions.common.duck.ParticleAccess;
import games.enchanted.eg_particle_interactions.common.mixin.accessor.client.ParticleAccessor;
import games.enchanted.eg_particle_interactions.common.particle.compat.CustomGeometryParticle;
import games.enchanted.eg_particle_interactions.common.util.MathHelpers;
import net.minecraft.class_2400;
import net.minecraft.class_3532;
import net.minecraft.class_4002;
import net.minecraft.class_5819;
import net.minecraft.class_638;
import net.minecraft.class_703;
import net.minecraft.class_707;
import net.minecraft.client.particle.*;
import org.jetbrains.annotations.Nullable;

public class FallingSpinningParticle extends CustomGeometryParticle {
    private float rotSpeed;
    protected float spinAcceleration;
    protected float maxSpinSpeed = 1f;
    private boolean transparency;

    protected FallingSpinningParticle(class_638 level, double x, double y, double z, double xSpeed, double ySpeed, double zSpeed, class_4002 spriteSet, float gravityMultiplier) {
        super(level, x, y, z, spriteSet.method_18139(level.field_9229));
        this.field_3844 = class_3532.method_32750(this.field_3840, 0.25F, 0.38F);;
        this.field_28786 = 1.0F;
        this.field_3852 = xSpeed + (Math.random() * 2.0 - 1.0) * 0.05000000074505806;
        this.field_3869 = ySpeed + (Math.random() * 2.0 - 1.0) * 0.05000000074505806;
        this.field_3850 = zSpeed + (Math.random() * 2.0 - 1.0) * 0.05000000074505806;
        this.field_3847 = (int)(16.0 / ((double)this.field_3840.method_43057() * 0.8 + 0.2)) + 2;
        this.spinAcceleration = (float)Math.toRadians(this.field_3840.method_43056() ? -5.0 : 5.0);
        this.roll = (float) Math.toRadians(this.field_3840.method_39332(0, 360));
        this.oRoll = this.roll;

        float particleSize = this.field_3840.method_43056() ? 0.07F : 0.08F;
        this.setScale(particleSize);
        this.method_3080(particleSize, particleSize);
        this.field_3844 *= gravityMultiplier;

        this.transparency = false;

        ((ParticleAccess) this).eg_particle_interactions$setBypassMovementCollisionCheck(true);
    }
    protected FallingSpinningParticle(class_638 level, double x, double y, double z, double xSpeed, double ySpeed, double zSpeed, class_4002 spriteSet, float gravityMultiplier, boolean transparency) {
        this(level, x, y, z, xSpeed, ySpeed, zSpeed, spriteSet, gravityMultiplier);
        this.transparency = transparency;
    }

    @Override
    public void method_3070() {
        this.rotSpeed += this.rotSpeed >= this.maxSpinSpeed ? 0 : (this.spinAcceleration / 2.0f);
        if(this.rotSpeed > this.maxSpinSpeed) this.rotSpeed = this.maxSpinSpeed;

        this.oRoll = this.roll;
        if( !this.field_3845 && !((ParticleAccessor) this).block_place_particle$getStoppedByCollision() ) {
            this.roll += this.rotSpeed / 6.5f;
        }

        this.field_3852 *= 0.949999988079071;
        this.field_3869 *= 0.8999999761581421;
        this.field_3850 *= 0.949999988079071;

        // if moving downwards
        if(this.field_3869 < 0 && ((ParticleAccess) this).eg_particle_interactions$getBypassMovementCollisionCheck()) {
            ((ParticleAccess) this).eg_particle_interactions$setBypassMovementCollisionCheck(false);
        }

        super.method_3070();
    }

    @Override
    protected ParticleLayer getParticleLayer() {
        return this.transparency ? ParticleLayer.TRANSLUCENT : ParticleLayer.OPAQUE;
    }

    public static class GenericLeafProvider implements class_707<class_2400> {
        private final class_4002 spriteSet;

        public GenericLeafProvider(class_4002 spriteSet) {
            this.spriteSet = spriteSet;
        }

        @Override
        public @Nullable class_703 createParticle(
            class_2400 options,
            class_638 level,
            double x,
            double y,
            double z,
            double xSpeed,
            double ySpeed,
            double zSpeed
            //? if minecraft: > 1.21.8 {
            , class_5819 random
            //?}
        ) {
            return new FallingSpinningParticle(level, x, y, z, xSpeed, ySpeed, zSpeed, spriteSet, 1);
        }
    }

    public static class SnowflakeProvider implements class_707<class_2400> {
        private final class_4002 spriteSet;

        public SnowflakeProvider(class_4002 spriteSet) {
            this.spriteSet = spriteSet;
        }

        @Override
        public @Nullable class_703 createParticle(
            class_2400 options,
            class_638 level,
            double x,
            double y,
            double z,
            double xSpeed,
            double ySpeed,
            double zSpeed
            //? if minecraft: > 1.21.8 {
            , class_5819 random
            //?}
        ) {
            FallingSpinningParticle particle = new FallingSpinningParticle(level, x, y, z, xSpeed, ySpeed, zSpeed, spriteSet, 0.4f, true);
            float particleSize = level.field_9229.method_43056() ? 0.10F : 0.12F;
            particle.setScale(particleSize);
            particle.method_3080(particleSize, particleSize);
            particle.spinAcceleration = 0;
            particle.maxSpinSpeed = 0;
            particle.roll = 0;
            particle.oRoll = 0;
            return particle;
        }
    }

    public static class RandomisedSizeMoreGravityProvider implements class_707<class_2400> {
        private final class_4002 spriteSet;

        public RandomisedSizeMoreGravityProvider(class_4002 spriteSet) {
            this.spriteSet = spriteSet;
        }

        @Override
        public @Nullable class_703 createParticle(
            class_2400 options,
            class_638 level,
            double x,
            double y,
            double z,
            double xSpeed,
            double ySpeed,
            double zSpeed
            //? if minecraft: > 1.21.8 {
            , class_5819 random
            //?}
        ) {
            FallingSpinningParticle particle = new FallingSpinningParticle(level, x, y, z, xSpeed, ySpeed, zSpeed, spriteSet, 2f);
            float particleSize = MathHelpers.randomBetween(0.08f, 0.12f);
            particle.setScale(particleSize);
            particle.method_3080(particleSize, particleSize);
            return particle;
        }
    }

    public static class PaleOakProvider implements class_707<class_2400> {
        private final class_4002 spriteSet;

        public PaleOakProvider(class_4002 spriteSet) {
            this.spriteSet = spriteSet;
        }

        @Override
        public @Nullable class_703 createParticle(
            class_2400 options,
            class_638 level,
            double x,
            double y,
            double z,
            double xSpeed,
            double ySpeed,
            double zSpeed
            //? if minecraft: > 1.21.8 {
            , class_5819 random
            //?}
        ) {
            FallingSpinningParticle particle = new FallingSpinningParticle(level, x, y, z, xSpeed, ySpeed, zSpeed, spriteSet, 0.6f);
            float particleSize = level.field_9229.method_43056() ? 0.1f : 0.15f;
            particle.setScale(particleSize);
            particle.method_3080(particleSize, particleSize);
            particle.maxSpinSpeed = 0.1f;
            particle.spinAcceleration = (float)Math.toRadians(level.field_9229.method_43056() ? -1.0 : 1.0);
            return particle;
        }
    }
}
